import decimal


# contexto actual, básico y extendido
print('(1) Contextos')
ctx = decimal.getcontext()
ctx_basic = decimal.BasicContext
ctx_extended = decimal.ExtendedContext
ctx_default = decimal.DefaultContext
print('Contexto actual\n', ctx)
print('Contexto básico\n', ctx_basic)
print('Contexto extendido\n', ctx_extended)
print('Contexto por defecto\n', ctx_default)
# contexto actual
print('\n(2) Contexto actual')
print('prec     =', ctx.prec)
print('Emin     =', ctx.Emin)
print('Emax     =', ctx.Emax)
print('capitals =', ctx.capitals)
print('clamp    =', ctx.clamp)
print('rounding =', ctx.rounding)
print('flags    =')
for f, v in ctx.flags.items():
    print(f'    {f}: {v}')
print('traps    =')
for t, v in ctx.traps.items():
    print(f'    {t}: {v}')


# operaciones y contexto
print('\n(3) Resultados según el contexto')
x=decimal.Decimal(1)
y=decimal.Decimal(3)
print('Contexto actual 1/3:', x / y)

# modificar contexto
ctx.prec = 8
print('Precisión [8]   1/3:', x / y)

# contexto local
with decimal.localcontext() as ctx_local:
    ctx_local.prec = 6
    ctx_local.rounding = decimal.ROUND_CEILING
    print('Contexto local. Precisión [6] 1/3:', x / y)
# fuera del with volvemos a utilizar el contexto actual
print('Contexto actual 1/3:', x / y)

# crear trampas (traps)
print('\n(4) Creación de trampas')
print('Trampa desactivada para operación no permitida')
print('Operación no permitida. x/y > 0.3:', x / y > 0.3)

print('\n(5) Activar trampa para operaciones no permitidas')
ctx.traps[decimal.FloatOperation] = True
try:
    print(x / y > 0.3)
except decimal.FloatOperation:
    print('Comparación no permitida')

# trampas (traps) e indicadores (flags)
print('\n(6) Desactivar trampa para operación inválida')
ctx.traps[decimal.InvalidOperation] = False
try:
    print('División por 0:', ctx.divide(decimal.Decimal(0), decimal.Decimal(0)))
except decimal.InvalidOperation:
    print('División no definida')
print('Trampa activada')
ctx.traps[decimal.InvalidOperation] = True
ctx.flags[decimal.InvalidOperation] = True
try:
    print('División por 0:', ctx.divide(decimal.Decimal(0), decimal.Decimal(0)))
except decimal.InvalidOperation:
    print('División no definida')
