from decimal import *


# establecer contexto
setcontext(ExtendedContext)
ctx = getcontext()
print('Contexto actual es el contexto extendido\n', ctx)

# valor absoluto
print('\nValor absoluto')
print('abs(-100):', ctx.abs(Decimal('-100')))
print('abs(-100.90):', ctx.abs(Decimal('-100.90')))

# máximos y mínimos
print('\nMáximos y mínimos')
print('_Máximo')
print('max(1, 2):', ctx.max(Decimal('1'), Decimal('2')))
print('max(1, -2):', ctx.max(1, -2))
print('max(1, NaN):', ctx.max(1, Decimal('NaN')))
print('_Mínimo')
print('min(1, 2):', ctx.min(Decimal('1'), Decimal('2')))
print('min(1, -2):', ctx.min(1, -2))
print('min(1, NaN):', ctx.min(1, Decimal('NaN')))
print('_Mínimo independiente del signo')
print('min_mag(1, -2):', ctx.min_mag(Decimal('1'), Decimal('-2')))

# paso a enteros
print('\nPaso a enteros. Redondeo según contexto')
print('Entero 1.5:', ctx.to_integral(Decimal('1.5')))
print('Entero 1.5:', ctx.to_integral_exact(Decimal('1.5')))
print('Entero 1.5:', ctx.to_integral_value(Decimal('1.5')))

# operaciones aritméticas
print('\nOperaciones aritméticas')
print('_Suma')
print('1.2+1.50:', ctx.add(Decimal('1.2'), Decimal('1.50')))
print('8+2:', ctx.add(Decimal(8), 2))
print('_Resta')
print('1.3-1.30:', ctx.subtract(Decimal('1.3'), Decimal('1.30')))
print('8-2:', ctx.subtract(Decimal(8), 2))
print('_Multiplicación')
print('1.2*3:', ctx.multiply(Decimal('1.20'), Decimal('3')))
print('123*-0:', ctx.multiply(Decimal('123'), Decimal('-0')))
print('123*1e15:', ctx.multiply(Decimal('123'), Decimal(1e15)))
print('_División')
print('1/3:', ctx.divide(Decimal('1'), Decimal('3')))
print('2/3:', ctx.divide(2, Decimal('3')))
print('3/3:', ctx.divide(3, 3))
print('_División entera')
print('1/3:', ctx.divide_int(Decimal('1'), Decimal('3')))
print('10/3:', ctx.divide_int(Decimal('10'), Decimal('3')))
print('_Resto')
print('10/3:', ctx.remainder(Decimal('10'), Decimal('3')))
print('_Parte entera y módulo')
print('10//3 10%3:', ctx.divmod(Decimal('10'), Decimal('3')))

# cambio de contexto
c = ctx.copy()          # copiar contexto
c.Emin = -999           # límite mínimo
c.Emax = 999            # límite máximo
print('\nModificado en contexto atributos Emin y Emax\n', c)

# potencias y raices
print('\nPotencias y raices')
print('_Potencia')
print('2**3:', c.power(Decimal('2'), Decimal('3')))
print('-2**3:', c.power(Decimal('-2'), Decimal('3')))
print('0**0:', c.power(0, 0))
print('infinito**0:', c.power(Decimal('Infinity'), Decimal('0')))
print('infinito**1:', c.power(Decimal('Infinity'), Decimal('1')))
print('-infinito**0:', c.power(Decimal('-Infinity'), Decimal('0')))
print('-infinito**1:', c.power(Decimal('-Infinity'), Decimal('1')))
print('_Exponencial')
print('e**0:', c.exp(0))
print('e**1:', c.exp(Decimal('1')))
print('e**100:', c.exp(Decimal('100')))
print('e**inf:', c.exp(Decimal('+Infinity')))
print('e**-inf:', c.exp(Decimal('-Infinity')))
print('_Raíz')
print('raíz 9:', c.sqrt(Decimal('9')))
print('raíz 2:', c.sqrt(Decimal('2')))

# logaritmos
print('\nLogaritmos')
print('_Naturales')
print('ln(1):', c.ln(1))
print('ln(2.71828183):', c.ln(Decimal('2.71828183')))
print('ln(2.68811714E+43):', c.ln(Decimal('2.68811714E+43')))
print('ln(+inf):', c.ln(Decimal('+Infinity')))
print('ln(-inf):', c.ln(Decimal('-Infinity')))
print('_Decimales')
print('log(0):', c.log10(0))
print('log(1):', c.log10(1))
print('log(10):', c.log10(10))
print('log(0.001):', c.log10(Decimal('0.001')))
print('log(+inf):', c.log10(Decimal('+Infinity')))

# valores por proximidad
print('\nValores por proximidad')
print('_Siguiente menor')
print('next_minus(1):', c.next_minus(1))
print('next_minus(inf):', c.next_minus(Decimal('Infinity')))
print('_Siguiente mayor')
print('next_plus(1):', c.next_plus(1))
print('next_plus(-inf):', c.next_plus(Decimal('-Infinity')))
print('_Más cercano')
print('next_toward(1, 2):', c.next_toward(Decimal('1'), Decimal('2')))
print('next_toward(1, 0):', c.next_toward(1, 0))
print('next_toward(0, 1):', c.next_toward(0, 1))

# consultas/comprobaciones
print('\nConsultas')
print('is_finite("1.23"):', c.is_finite(Decimal('1.23')))
print('is_finite("Inf"):', c.is_finite(Decimal('Inf')))
print('is_infinite("1.23"):', c.is_infinite(Decimal('1.23')))
print('is_infinite("Inf"):', c.is_infinite(Decimal('Inf')))
print('is_normal("1.23"):', c.is_normal(Decimal('1.23')))
print('is_normal("0.0"):', c.is_normal(Decimal('0.0')))
print('is_signed("1.23"):', c.is_signed(Decimal('1.23')))
print('is_signed("-1.23"):', c.is_signed(Decimal('-1.23')))
print('is_zero("0"):', c.is_zero(Decimal('0')))
print('is_zero("-1.23"):', c.is_zero(Decimal('-1.23')))
